package net.pleus.bddemo.streaming;

import org.apache.kafka.common.protocol.types.Schema;
import org.apache.spark.streaming.kafka010.KafkaUtils;

import org.apache.commons.io.IOUtils;

import java.text.SimpleDateFormat;
import java.util.*;

import org.apache.spark.sql.Encoder;
import org.apache.spark.sql.Encoders;
import org.apache.avro.specific.SpecificDatumReader;
import org.apache.kafka.clients.consumer.ConsumerConfig;
import org.apache.kafka.clients.consumer.ConsumerRecord;
import org.apache.spark.streaming.Durations;
import org.apache.spark.streaming.api.java.*;
import org.apache.spark.streaming.kafka010.*;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.api.java.function.*;
import org.apache.spark.sql.Dataset;
import org.apache.spark.sql.SaveMode;
import org.apache.spark.sql.SparkSession;

public class JobsProcessor
{
    private static final String ZOOKEEPER_SERVER = "localhost:9092";
    private static final int BATCH_INTERVAL = 10;
    private static final String TOPIC = "jobs";

    private static final String DATETIME_FORMAT ="yyyyMMdd_HHmmssSSS";
    private static final SimpleDateFormat formatter = new SimpleDateFormat(DATETIME_FORMAT);

    private static final Encoder<Row> ENCODER = Encoders.bean(Row.class);

    public static void main(String[] args) throws Exception{

        try{

            System.out.println("Starting streaming consumer for " + TOPIC + " ...");

            // Create configuration
            Map<String, Object> kafkaParams = new HashMap<>();
            kafkaParams.put(ConsumerConfig.BOOTSTRAP_SERVERS_CONFIG, ZOOKEEPER_SERVER);
            kafkaParams.put(ConsumerConfig.KEY_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");
            kafkaParams.put(ConsumerConfig.VALUE_DESERIALIZER_CLASS_CONFIG, "org.apache.kafka.common.serialization.StringDeserializer");
            kafkaParams.put(ConsumerConfig.GROUP_ID_CONFIG,"group1");
            kafkaParams.put(ConsumerConfig.AUTO_OFFSET_RESET_CONFIG,"latest");
            kafkaParams.put(ConsumerConfig.ENABLE_AUTO_COMMIT_CONFIG,true);
            Collection<String> topics = Arrays.asList(TOPIC);

            // Create spark session
            SparkSession spark = SparkSession.builder()
                .appName("Jobs Streaming Processor")
                .getOrCreate();

            // Create streaming context
            JavaSparkContext javaSparkContext = new JavaSparkContext(spark.sparkContext());
            JavaStreamingContext javaStreamingContext = new JavaStreamingContext(javaSparkContext, Durations.seconds(BATCH_INTERVAL));
            
            // Create stream
            JavaInputDStream<ConsumerRecord<String, String>> stream = KafkaUtils.createDirectStream(
                javaStreamingContext,
                LocationStrategies.PreferConsistent(),
                ConsumerStrategies.<String, String>Subscribe(topics, kafkaParams)
            );

            // Transform records
            JavaDStream<Row> rows = stream.map(record -> new Row(record.value()));

            // Save data
            rows.foreachRDD( javaRdd -> {
                Dataset<Row> dataset = spark.createDataset(javaRdd.rdd(), ENCODER);
                dataset.write()
                       .mode(SaveMode.Append)
                       .option("header",false)
                       .option("quote", "\u0000")
                       .csv("hdfs://localhost:9000/bddemo/data/");
            });

            // Start processing
            javaStreamingContext.start();
            javaStreamingContext.awaitTermination();            

        }
        catch(Throwable t){
            // Dump error
            t.printStackTrace();
        }
    }
}